import { AnimatedSprite, Container, Texture, Ticker } from 'pixi.js'
import { Viewport } from 'pixi-viewport'
import { idle_animations, running_animations } from '@/model/GameElement/animationTextures'

/**
 * Class representing a Character in the game.
 * @extends {Container} The PIXI.js Container class.
 */
export class Character extends Container {
  /**
   * The size of each step the character takes.
   * @type {number}
   */
  protected STEP_SIZE : number = 32

  /**
   * The speed of the idle animation.
   * @type {number}
   */
  protected IDLE_ANIMATION_SPEED = 0.15

  /**
   * The speed of the running animation.
   * @type {number}
   */
  protected RUNNING_ANIMATION_SPEED = 0.20

  /**
   * The sprite of the player character.
   * @type {AnimatedSprite}
   */
  protected readonly player_sprite : AnimatedSprite;

  /**
   * The unique identifier of the character.
   * @type {string}
   */
  public uid: string

  /**
   * The idle animations for the character. The index corresponds to the direction (0 = front, 1 = back, 2 = left, 3 = right).
   * @type {Array<Array<Texture>>}
   */
  protected idle_animations : Array<Array<Texture>>

  /**
   * The running animations for the character. The index corresponds to the direction (0 = front, 1 = back, 2 = left, 3 = right).
   * @type {Array<Array<Texture>>}
   */
  protected running_animations : Array<Array<Texture>>

  /**
   * The last direction the character was facing.
   * @type {number}
   */
  protected lastDirection : number = 0

  /**
   * The viewport the character is rendered in.
   * @type {Viewport}
   */
  protected viewPort : Viewport

  /**
   * The ticker for the game loop.
   * @type {Ticker}
   */
  protected ticker: Ticker

  /**
   * Whether the character is idle or not.
   * @type {boolean}
   */
  protected isIdle: boolean = false

  /**
   * The constructor for the Character class.
   * @param {string} uid - The unique identifier for the character.
   * @param {number} x - The initial x position of the character.
   * @param {number} y - The initial y position of the character.
   * @param {Viewport} viewPort - The viewport the character is rendered in.
   */
  constructor(uid : string, x : number,y: number, viewPort : Viewport ) {
    super()
    this.viewPort = viewPort
    this.idle_animations = []
    this.running_animations = []
    for (let i = 0; i < idle_animations.length; i++) {
      this.idle_animations[i] = idle_animations[i].map((stringy) => Texture.from(stringy))
      this.running_animations[i] = running_animations[i].map((stringy) => Texture.from(stringy))
    }
    this.uid = uid
    this.player_sprite = new AnimatedSprite(this.idle_animations[0]);
    this.player_sprite.animationSpeed = 0.15
    this.player_sprite.play()
    this.player_sprite.anchor.set(0.5, 1);
    this.player_sprite.scale.set(2,2);
    this.player_sprite.position.set(x, y);
    this.player_sprite.eventMode = 'static';
    this.player_sprite.cursor = 'pointer';
    this.ticker = Ticker.shared;
    viewPort.addChild(this.player_sprite)
  }
}