"use strict";
import txt from "../../../assets/map/collision_grid.txt"
import {Socket} from "socket.io-client";
import {Easing, Group, Tween} from "tweedle.js";
import {Viewport} from "pixi-viewport";
import { Character } from '@/model/GameElement/Character/Character'
import type { Case } from '@/model/GameElement/map'
import { getTileMapCollisionFromTxt } from '@/model/GameElement/map'
import { Ticker } from 'pixi.js'


/**
 * Class representing a Hero in the game, which extends the Character class.
 */
export class Hero extends Character {
    /**
     * The socket used for communication with the server.
     * @type {Socket}
     */
    private socket: Socket

    /**
     * A flag indicating whether the hero can trigger a new movement.
     * @type {boolean}
     */
    public canTriggerNewMovement  : boolean = true

    /**
     * The collision map of the game.
     * @type {Array<Array<Case>>}
     */
    private collisionMap: Array<Array<Case>>

    /**
     * The current position of the hero on the collision map.
     * @type {{x: number, y:number}}
     */
    private positionOnCollisionMap : {x: number, y:number}

    /**
     * The last key pressed by the user.
     * @type {string}
     */
    private key: string = '';

    /**
     * A map of keys and their pressed state.
     * @type {Map<string, boolean>}
     */
    private keys: Map<string, boolean> = new Map()

    /**
     * The constructor for the Hero class.
     * @param {Socket} socket - The socket used for communication with the server.
     * @param {Viewport} viewPort - The viewport in which the hero is rendered.
     * @param {string} uid - The unique identifier for the hero.
     */
    constructor(socket: Socket, viewPort: Viewport, uid: string) {
        super(uid, 512, 256, viewPort)
        this.positionOnCollisionMap = {x: 32,y: 16}
        this.collisionMap = getTileMapCollisionFromTxt(txt)
        console.log(this.collisionMap)
        this.socket = socket;
        document.addEventListener("keydown", this.onKeyPressed.bind(this));
        document.addEventListener("keyup", this.onKeyUp.bind(this));
        viewPort.follow(this.player_sprite)
        Ticker.shared.add(this.update, this);
    }

    /**
     * Event handler for the keydown event.
     * @param {KeyboardEvent} e - The event object.
     */
    private onKeyPressed(e: KeyboardEvent){
        if(e.key === 'w' || e.key === 's' || e.key === 'd' || e.key === 'a'){
            this.keys.set(e.code, true)
            this.key = e.code;
        }
    }

    /**
     * Event handler for the keyup event.
     * @param {KeyboardEvent} e - The event object.
     */
    private onKeyUp(e: KeyboardEvent):void {
        if(e.key === 'w' || e.key === 's' || e.key === 'd' || e.key === 'a'){
            this.player_sprite.textures = this.idle_animations[this.lastDirection];
            this.player_sprite.animationSpeed = this.IDLE_ANIMATION_SPEED
            this.player_sprite.play()
            this.isIdle = true
            this.keys.set(e.code, false)
            let atLeastOneKeyPressed = false
            for ( const [key, value] of this.keys.entries()){
                if(value){
                    this.key = key;
                    atLeastOneKeyPressed = true;
                }
            }
            if (!atLeastOneKeyPressed) {
                this.key = ''
            }
        }
    }

    /**
     * Handles the keydown event to move the player sprite.
     */
    private onKeyDown(): void {
        let currentDirection = 0
        if (this.canTriggerNewMovement) {
            let isMoving: boolean = false
            this.canTriggerNewMovement = false
            let direction: { x: number, y: number, type: number }
            switch (this.key) {
                case 'KeyW':
                    currentDirection = 1
                    direction = {
                        x: this.player_sprite.x,
                        y: this.player_sprite.position.y - this.STEP_SIZE,
                        type: currentDirection
                    }
                    isMoving = true
                    break;
                case 'KeyS':
                    currentDirection = 0
                    direction = {
                        x: this.player_sprite.x,
                        y: this.player_sprite.position.y + this.STEP_SIZE,
                        type: currentDirection
                    }
                    isMoving = true
                    break;
                case 'KeyA':
                    currentDirection = 2
                    direction = {
                        x: this.player_sprite.x - this.STEP_SIZE,
                        y: this.player_sprite.position.y,
                        type: currentDirection
                    }
                    isMoving = true
                    break;
                case 'KeyD':
                    currentDirection = 3
                    direction = {
                        x: this.player_sprite.x + this.STEP_SIZE,
                        y: this.player_sprite.position.y,
                        type: currentDirection
                    }
                    isMoving = true
                    break;
                default:
                    direction = {
                        x: this.player_sprite.x,
                        y: this.player_sprite.position.y,
                        type: 1
                    }
                    break;
            }

            if(isMoving && !this.checkForObstacles(direction.type)){
                if(this.lastDirection !== currentDirection || this.isIdle){
                    this.isIdle = false
                    this.player_sprite.textures = this.running_animations[currentDirection];
                    this.player_sprite.animationSpeed = this.RUNNING_ANIMATION_SPEED
                    this.player_sprite.play()
                }
                this.lastDirection = currentDirection
                console.log("Player is Moving !")
                this.movePlayer(direction)
            } else{
                this.canTriggerNewMovement = true
            }
        }
    }

    /**
     * Moves the player sprite to a new position.
     * @param {Object} position - An object containing the x and y coordinates for the new position.
     */
    private movePlayer(position: {x:number,y:number, type:number}){
        this.socket.emit("player moving", {uid: this.uid, x:position.x, y: position.y, direction: this.lastDirection})
        new Tween(this.player_sprite.position)
          .to({
              x: position.x,
              y: position.y
          }, 200)
          .easing(Easing.Linear.None)
          .start()
          .onComplete(()=>{
              this.player_sprite.position.set(
                this.player_sprite.x = Math.round(this.player_sprite.x / 8) * 8,
                this.player_sprite.y = Math.round(this.player_sprite.y / 8) * 8
              )
              this.canTriggerNewMovement = true
          })

        const tweenfct = () =>{
            Group.shared.update()
        }

        this.ticker.add(tweenfct)
    }

    /**
     * Checks for obstacles in the given direction.
     * @param {number} direction - The direction to check for obstacles.
     * @returns {Boolean} - Returns true if there is an obstacle, false otherwise.
     */
    private checkForObstacles(direction: number):Boolean{
        switch (direction){
            case 0:
                if(!this.collisionMap[this.positionOnCollisionMap.y + 1][this.positionOnCollisionMap.x].isObstacle){
                    this.positionOnCollisionMap = {x: this.positionOnCollisionMap.x, y: this.positionOnCollisionMap.y + 1}
                    return false
                }
                break;
            case 1:
                if(!this.collisionMap[this.positionOnCollisionMap.y - 1][this.positionOnCollisionMap.x].isObstacle){
                    this.positionOnCollisionMap = {x: this.positionOnCollisionMap.x, y: this.positionOnCollisionMap.y - 1}
                    return false
                }
                break;
            case 2:
                if(!this.collisionMap[this.positionOnCollisionMap.y][this.positionOnCollisionMap.x - 1].isObstacle){
                    this.positionOnCollisionMap = {x: this.positionOnCollisionMap.x - 1, y: this.positionOnCollisionMap.y}
                    return false
                }
                break;
            case 3:
                if(!this.collisionMap[this.positionOnCollisionMap.y][this.positionOnCollisionMap.x + 1].isObstacle){
                    this.positionOnCollisionMap = {x: this.positionOnCollisionMap.x + 1, y: this.positionOnCollisionMap.y}
                    return false
                }
                break;
            default:
                return true
        }
        return true
    }

    /**
     * Updates the game state.
     * This is useful to remove the input lag on eventListener
     * @param {number} deltaTime - The time elapsed since the last frame.
     */
    private update(deltaTime: number): void {
        this.onKeyDown()
    }

    /**
     * Sets the canTriggerNewMovement property.
     * This will be use to prevent player from moving on specific actions
     * @param {boolean} value - The new value for the canTriggerNewMovement property.
     */
    public setCanTriggerNewMovement(value: boolean){
        console.log("User can Move ?", value)
        this.canTriggerNewMovement = value
    }
}