import { Easing, Group, Tween } from 'tweedle.js'
import { Viewport } from 'pixi-viewport'
import { GlowFilter } from 'pixi-filters'
import { useUiStore } from '@/stores/uiStore'
import { Character } from '@/model/GameElement/Character/Character'
import { useUserStore } from '@/stores/userStore'

/**
 * Class representing a Player in the game, which extends the Character class.
 */
export class Player extends Character {
  /**
   * A flag indicating whether the player is selected.
   * @type {boolean}
   */
  private isSelected: boolean

  /**
   * The UI store used for managing UI state.
   * @type {any}
   */
  private uiStore: any

  /**
   * The user store used for managing user state.
   * @type {any}
   */
  private userStore: any

  /**
   * The constructor for the Player class.
   * @param {string} uid - The unique identifier for the player.
   * @param {number} x - The initial x position of the player.
   * @param {number} y - The initial y position of the player.
   * @param {Viewport} viewPort - The viewport in which the player is rendered.
   */
  constructor(uid: string, x: number, y: number, viewPort: Viewport) {
    super(uid, x, y, viewPort)
    this.uid = uid
    this.isSelected = false
    this.player_sprite.on('mouseover', this.glowPlayer.bind(this))
    this.player_sprite.on('mouseleave', this.removeGlowPlayer.bind(this))
    this.player_sprite.on('click', this.selectPlayer.bind(this))
    this.uiStore = useUiStore()
    this.userStore = useUserStore()
  }

  /**
   * Removes the player from the scene.
   */
  public removePlayerFromScene() {
    this.viewPort.removeChild(this.player_sprite)
  }

  /**
   * Applies a glow effect to the player sprite.
   * This is typically called when the mouse hovers over the player.
   */
  private glowPlayer() {
    this.player_sprite.filters = [new GlowFilter({ distance: 15, outerStrength: 2 })]
  }

  /**
   * Selects the player and opens the player profile in the UI store.
   * This is typically called when the player is clicked.
   */
  private selectPlayer() {
    this.isSelected = true
    this.uiStore.openPlayerProfile()
    console.log("Selected player with uid: ", this.uid)
    this.userStore.setSelectedUserId(this.uid)
  }

  /**
   * Removes the glow effect from the player sprite.
   * This is typically called when the mouse leaves the player.
   */
  private removeGlowPlayer() {
    this.player_sprite.filters = null
  }

  /**
   * Moves the player sprite to a new position asynchronously.
   *
   * @param position - An object containing the x and y coordinates for the new position.
   * @returns A promise that resolves when the movement is complete.
   */
  public async movePlayer(position: { x: number, y: number, type: number }) {
    this.lastDirection = position.type
    this.player_sprite.textures = this.running_animations[this.lastDirection]
    this.player_sprite.animationSpeed = this.RUNNING_ANIMATION_SPEED
    this.player_sprite.play()

    const deltaX = position.x - this.player_sprite.x
    const deltaY = position.y - this.player_sprite.y
    // Create a new tween for the player sprite's position
    new Tween(this.player_sprite.position).to({
      x: position.x,
      y: position.y
    }, 200 * (Math.sqrt((deltaX * deltaX) + (deltaY * deltaY)) / 32)
    ) // Set the duration of the tween to 300ms
      .easing(Easing.Sinusoidal.Out) // Use the Back InOut easing function
      .start() // Start the tween
      .onComplete(() => {
        // Normalize the position of the player sprite
        this.player_sprite.position.set(
          this.player_sprite.x = Math.round(this.player_sprite.x / 8) * 8,
          this.player_sprite.y = Math.round(this.player_sprite.y / 8) * 8)
        this.player_sprite.textures = this.idle_animations[this.lastDirection]
        this.player_sprite.animationSpeed = this.IDLE_ANIMATION_SPEED
        this.player_sprite.play() // Resolve the promise when the movement is over
      })
    // Define a function to update the shared group
    const tweenfct = () => {
      Group.shared.update()
    }
    // Add the function to the ticker
    this.ticker.add(tweenfct)
  }

}