import sprite from "../../assets/table.png"
import { Sprite } from "pixi.js"
import { Viewport } from "pixi-viewport"
import { GlowFilter } from 'pixi-filters'
import { useUiStore } from '@/stores/uiStore'
import { useChatStore } from '@/stores/chatStore'

/**
 * Class representing a chat table in the game.
 */
export class ChatTable {
  /**
   * The name of the chat room associated with this table.
   * @type {string}
   */
  public chatRoomName: string

  /**
   * The viewport in which this table is rendered.
   * @type {Viewport}
   */
  private viewPort: Viewport

  /**
   * The sprite representing the table in the game.
   * @type {Sprite}
   */
  private table_sprite: Sprite

  /**
   * The UI store used for managing UI state.
   * @type {any}
   */
  private uiStore: any

  /**
   * The chat store used for managing chat state.
   * @type {any}
   */
  private chatStore: any

  /**
   * Create a new chat table.
   * @param {string} name - The name of the chat room associated with this table.
   * @param {number} x - The x-coordinate at which to place the table.
   * @param {number} y - The y-coordinate at which to place the table.
   * @param {Viewport} viewPort - The viewport in which to render the table.
   * @param {Array<ChatTable>} chatTables - The array of all chat tables in the game.
   */
  constructor(name: string, x: number, y: number, viewPort: Viewport, chatTables) {
    this.chatRoomName = name
    this.table_sprite = Sprite.from(sprite);
    this.table_sprite.anchor.set(0.5, 0.5);
    this.table_sprite.scale.set(2, 2);
    this.table_sprite.position.set(x, y);
    this.table_sprite.eventMode = 'static';
    this.table_sprite.cursor = 'pointer';
    this.table_sprite.on('mouseover', this.glowElement.bind(this));
    this.table_sprite.on('mouseleave', this.removeGlow.bind(this))
    this.table_sprite.on('click', this.selectTable.bind(this))
    this.viewPort = viewPort
    this.viewPort.addChild(this.table_sprite)
    this.uiStore = useUiStore()
    this.chatStore = useChatStore()
    chatTables.push(this)
  }

  /**
   * Apply a glow effect to the table sprite.
   * This is typically called when the mouse hovers over the table.
   */
  private glowElement() {
    this.table_sprite.filters = [new GlowFilter({ distance: 15, outerStrength: 5 })]
  }

  /**
   * Select this table and join its associated chat room.
   * This is typically called when the table is clicked.
   */
  private selectTable() {
    this.chatStore.setChatRoom(this.chatRoomName)
    this.chatStore.joinChatRoom()
    this.uiStore.toggleChatDialog()
  }

  /**
   * Remove the glow effect from the table sprite.
   * This is typically called when the mouse leaves the table.
   */
  private removeGlow() {
    this.table_sprite.filters = null
  }
}