import { GlowFilter } from "pixi-filters"
import sprite from "../../assets/sign.png"
import type { Viewport } from "pixi-viewport"
import { Sprite } from "pixi.js"
import { useUiStore } from "@/stores/uiStore"
import type { InstitutionData } from '@/model/InstitutionData'
import { useInstitutionStore } from '@/stores/institutionStore'

/**
 * Class representing a sign for books in the game.
 */
export class InstitutionSign {
    /**
     * The name of the sign.
     * @type {string}
     */
    public institution: InstitutionData

    /**
     * The viewport in which this sign is rendered.
     * @type {Viewport}
     */
    private viewPort: Viewport

    /**
     * The sprite representing the sign in the game.
     * @type {Sprite}
     */
    private signSprite: Sprite

    /**
     * The UI store used for managing UI state.
     * @type {any}
     */
    private uiStore: any

    /**
     * The institution store used for managing institution state.
     * @type {any}
     */
    private institutionStore: any

    /**
     * Create a new sign.
     * @param {string} institution - The name of the institution.
     * @param {number} x - The x-coordinate at which to place the sign.
     * @param {number} y - The y-coordinate at which to place the sign.
     * @param {Viewport} viewPort - The viewport in which to render the sign.
     */
    constructor(institution: InstitutionData, x: number, y: number, viewPort: Viewport) {
        this.institution = institution
        this.signSprite = Sprite.from(sprite)
        this.signSprite.anchor.set(0.5, 0.5)
        this.signSprite.position.set(x, y)
        this.signSprite.scale.set(2, 2)
        this.signSprite.eventMode = 'static'
        this.signSprite.cursor = 'pointer'
        this.signSprite.on('mouseover', this.glowElement.bind(this))
        this.signSprite.on('mouseleave', this.removeGlow.bind(this))
        this.signSprite.on('click', this.selectSign.bind(this))
        this.viewPort = viewPort
        this.viewPort.addChild(this.signSprite)
        this.uiStore = useUiStore()
        this.institutionStore = useInstitutionStore()
    }

    /**
     * Apply a glow effect to the sign sprite.
     * This is typically called when the mouse hovers over the sign.
     */
    private glowElement() {
        this.signSprite.filters = [new GlowFilter({ distance: 15, outerStrength: 2 })]
    }

    /**
     * Remove the glow effect from the sign sprite.
     * This is typically called when the mouse leaves the sign.
     */
    private removeGlow() {
        this.signSprite.filters = null
    }

    /**
     * Select this sign and toggle the book sign in the UI store.
     * This is typically called when the sign is clicked.
     */
    private selectSign() {
        this.institutionStore.setSelectedInstitution(this.institution)
        this.uiStore.toggleBookSign()
    }
}