import sprite from '../../assets/library.png'
import { Sprite } from 'pixi.js'
import { Viewport } from 'pixi-viewport'
import { GlowFilter } from 'pixi-filters'
import { useUiStore } from '@/stores/uiStore'
import { useLibraryStore } from '@/stores/libraryStore'
import type { InstitutionData } from '../InstitutionData'
import { useInstitutionStore } from '@/stores/institutionStore'

/**
 * Class representing a library dialog.
 */
export class LibraryAction {
  /**
   * The name of the library.
   * @type {string}
   */
  public institution: InstitutionData

  /**
   * The viewport in which this table is rendered.
   * @type {Viewport}
   */
  private viewPort: Viewport

  /**
   * The sprite representing the table in the game.
   * @type {Sprite}
   */
  private library_sprite: Sprite

  /**
   * The UI store used for managing UI state.
   * @type {any}
   */
  private uiStore: any

  /**
   * The chat store used for managing chat state.
   * @type {any}
   */
  private libraryStore: any

  /**
   * The institution store used for managing institution state.
   * @type {any}
   */
  private institutionStore: any

  /**
   * Create a new chat table.
   * @param {string} name - The name of the chat room associated with this table.
   * @param {number} x - The x-coordinate at which to place the table.
   * @param {number} y - The y-coordinate at which to place the table.
   * @param {Viewport} viewPort - The viewport in which to render the table.
   * @param {Array<ChatTable>} chatTables - The array of all chat tables in the game.
   */
  constructor(
    institution: InstitutionData,
    x: number,
    y: number,
    viewPort: Viewport,
    chatTables,
  ) {
    this.institution = institution
    this.library_sprite = Sprite.from(sprite)
    this.library_sprite.anchor.set(0.5, 0.5)
    this.library_sprite.scale.set(2, 2)
    this.library_sprite.position.set(x, y)
    this.library_sprite.eventMode = 'static'
    this.library_sprite.cursor = 'pointer'
    this.library_sprite.on('mouseover', this.glowElement.bind(this))
    this.library_sprite.on('mouseleave', this.removeGlow.bind(this))
    this.library_sprite.on('click', this.selectLibrary.bind(this))
    this.viewPort = viewPort
    this.viewPort.addChild(this.library_sprite)
    this.uiStore = useUiStore()
    this.libraryStore = useLibraryStore()
    this.institutionStore = useInstitutionStore()
    chatTables.push(this)
  }

  /**
   * Apply a glow effect to the table sprite.
   * This is typically called when the mouse hovers over the table.
   */
  private glowElement() {
    this.library_sprite.filters = [new GlowFilter({ distance: 15, outerStrength: 5 })]
  }

  /**
   * Select this table and join its associated chat room.
   * This is typically called when the table is clicked.
   */
  private selectLibrary() {
    // find institution with the same name
    this.institutionStore.setSelectedInstitution(this.institution)
    this.uiStore.toggleLibrary()
  }

  /**
   * Remove the glow effect from the table sprite.
   * This is typically called when the mouse leaves the table.
   */
  private removeGlow() {
    this.library_sprite.filters = null
  }
}
