import { defineStore } from 'pinia'
import type { Message } from '@/model/Message'
import { io } from 'socket.io-client'
import { useUserStore } from '@/stores/userStore'
import { doc, setDoc, query, where, onSnapshot, getDocs, collection, QuerySnapshot } from 'firebase/firestore'
import { db } from '@/stores/utils/firebase'
import type { Hero } from '@/model/GameElement/Character/hero'
import type { ChatData} from '@/model/ChatData'
import type { UserData } from '@/model/userData'
import type { Unsubscribe } from '@firebase/firestore'
import { v4 as uuidv4 } from 'uuid'

// Establish a connection to the game service using socket.io
const socket = io("https://game-service-993049008220.europe-west4.run.app", {autoConnect: false})



/**
 * Defines a store named 'chatStore' using the 'defineStore' function from 'pinia'.
 * This store is used to manage the state and actions related to the chat feature.
 */
export const useChatStore = defineStore('chatStore', {
  /**
   * Defines the initial state of the store.
   */
  state: () => {
    return {
      chatRoom: '' as string | undefined, // The current chat room.
      chatMessages: [] as Message[], // The messages in the current chat room.
      scrollControler: null as any, // The scroll controller for the chat window.
      hero: null as Hero | null,
      privateMessageSub: null as Unsubscribe | null,
      isInPrivateChat: false,
      privateChat: null as ChatData | null,
      privateChats: [] as ChatData[]// The hero character associated with the user.
    }
  },
  actions: {
    /**
     * Joins a chat room.
     */
    joinChatRoom(){
      this.isInPrivateChat = false
      // Log the current movement state of the hero
      console.log(this.hero?.canTriggerNewMovement)
      console.log("Joining chat room named : ", this.chatRoom)
      // Toggle the movement state of the hero
      this.hero?.setCanTriggerNewMovement(!this.hero?.canTriggerNewMovement)
      // Connect to the socket
      socket.connect()
      // Listen for new messages in the chat room
      socket.on(`${this.chatRoom}`, (msg)=>{
        console.log("New message received !", msg)
        console.log(this.getChatMessages)
        // Add the new message to the chat messages
        this.chatMessages.push(msg)
        // Scroll to the new message
        this.scrollControler(400, {
          container: '#goto-container',
          duration: 200,
          easing: 'easeInOutCubic',
          offset: 450,})
      })
    },
    joinPrivateChatRoom(privateChatRoomUuid:string){
      this.chatRoom = privateChatRoomUuid
      console.log(this.hero?.canTriggerNewMovement)
      console.log("Joining chat room named : ", this.chatRoom)
      // Toggle the movement state of the hero
      this.hero?.setCanTriggerNewMovement(!this.hero?.canTriggerNewMovement)
      this.privateMessageSub = onSnapshot(collection(db, `chats/${this.chatRoom}/messages`), (snapshot: QuerySnapshot) => {
        snapshot.docChanges().forEach((change) => {
          if (change.type === 'added') {
            let privateMessage = change.doc.data() as Message
            console.log(privateMessage)
            this.chatMessages.push(privateMessage)
            this.scrollControler(400, {
              container: '#goto-container',
              duration: 200,
              easing: 'easeInOutCubic',
              offset: 450,})
          }
        });
      });
      this.isInPrivateChat = true
    },
    async getAllPrivateChatRoom(){
      // Get the user store instance
      const userStore = useUserStore()
      if (userStore.user === null) {
        console.log("User not logged in")
        return
      }
      const chatsRef = collection(db, 'users', userStore.user?.uid, 'chats')
      const querySnapshot = await getDocs(chatsRef)
      this.privateChats = querySnapshot.docs.map((doc) => {return doc.data() as ChatData} )
      console.log("Number of private chats = " + this.privateChats.length)
    },

    async setNewChatRoom(hero: UserData,selectedUser: UserData){
      console.log("Create new private chat room");
      if (!hero || !selectedUser) return

      // 1. Vérifier si un chat existe déjà pour "hero" ayant otherUsername = selectedUser.username
      const chatsRef = collection(db, 'users', hero.uid, 'chats')
      const existingQuery = query(chatsRef, where('otherUsername', '==', selectedUser.username))
      const querySnapshot = await getDocs(existingQuery)

      if (!querySnapshot.empty) {
        // On considère qu'il n'y a qu'un seul chat par duo, donc on récupère le premier
        const existingDoc = querySnapshot.docs[0]
        const existingData = existingDoc.data() as { chatUUID: string }
        this.chatRoom = existingData.chatUUID
        console.log("Chat déjà existant. ID:", this.chatRoom)
      } else {

        const chatUUID = uuidv4()
        await setDoc(
          doc(collection(db, `users/${hero.uid}/chats`), chatUUID),
          {
            chatUUID: chatUUID,
            otherUsername: selectedUser.username
          }
        )

        // Pour la personne 2 (selectedUser)
        await setDoc(
          doc(collection(db, `users/${selectedUser.uid}/chats`), chatUUID),
          {
            chatUUID: chatUUID,
            otherUsername: hero.username
          }
        )
        this.chatRoom = chatUUID
        console.log(`Nouveau chat créé : ${chatUUID}`)
      }
      this.joinPrivateChatRoom(<string>this.chatRoom)
    },
    /**
     * Sends a message to the chat room.
     * @param {string} message - The message to send.
     */
    async sendMessage(message: string){
      const userStore = useUserStore()

      if(this.isInPrivateChat){
        const messageUUID = uuidv4()
        await setDoc(doc(collection(db, `chats/${this.chatRoom}/messages`),messageUUID,),{
          uid: userStore.uid,
          message: message,
          sender: userStore.username,
          timestamp: Date.now()
        })
      } else {
        const ioMessage = {chatroom: this.chatRoom, message: message, sender: userStore.username, uid: userStore.uid, timestamp: Date.now() }
        console.log("chatStore: send message", ioMessage)
        // Emit the message to the socket
        socket.emit(`chat message`, ioMessage)
      }
    },
    /**
     * Leaves the current chat room.
     */
    leaveChatRoom(){
      // Enable movement for the hero
      this.hero?.setCanTriggerNewMovement(true)

      this.isInPrivateChat = false
      if (this.privateMessageSub !== null) {
        this.privateMessageSub()
      }
      // Disconnect from the socket
      socket.disconnect()
      // Remove all listeners from the socket
      socket.removeAllListeners()
      // Clear the chat messages
      this.chatMessages = []
      // Clear the chat room
      this.chatRoom = ''
    },
    /**
     * Sets the current chat room.
     * @param {string} chatRoom - The chat room to set.
     */
    setChatRoom(chatRoom: string){
      this.chatRoom = chatRoom
    },
    /**
     * Sets the scroll controller.
     * @param {any} scrollController - The scroll controller to set.
     */
    setScrollController(scrollController: any){
      this.scrollControler = scrollController
    },
    /**
     * Sets the hero character.
     * @param {Hero} hero - The hero to set.
     */
    setHero(hero: Hero){
      this.hero = hero
    }
  },
  getters: {
    /**
     * Gets the current chat room.
     * @returns {string} - The current chat room.
     */
    getChatRoom():string{
      return <string>this.chatRoom;
    },
    /**
     * Gets the chat messages.
     * @returns {Message[]} - The chat messages.
     */
    getChatMessages():Message[]{
      return this.chatMessages;
    }
  }
})