import type { LibraryData } from '@/model/LibraryData'
import { defineStore } from 'pinia'
import { db, storage } from '@/stores/utils/firebase'
import {
  addDoc,
  collection,
  collectionGroup,
  deleteDoc,
  doc,
  getDoc,
  getDocs,
  query,
  Timestamp,
  where
} from '@firebase/firestore'
import { deleteObject, getDownloadURL, ref, uploadBytes } from 'firebase/storage'
import type { UserData } from '@/model/userData'
import type { DocumentData } from '@/model/DocumentData'
import { withLoading } from '@/stores/utils'

/**
 * Defines a store named 'LibraryStore' using the 'defineStore' function from 'pinia'.
 * This store is used to manage the state and actions related to libraries.
 */
export const useLibraryStore = defineStore('LibraryStore', {
  state: () => {
    return {
      selectedLibrary: null as LibraryData | null,
      libraries: [] as LibraryData[],
      libraryName: '' as string | undefined
    }
  },
  actions: {
    openLibraryByInstitutionId(institutionId: string) {
      const library = this.libraries.find((lib) => lib.InstitutionID === institutionId)
      if (library != null) {
        this.openLibrary(library)
      } else {
        console.error('Library not found for institution ID:', institutionId)
      }
    },
    openLibrary(library: LibraryData) {
      this.selectedLibrary = library
    },
    closeLibrary() {
      this.selectedLibrary = null
    },
    async fetchLibraries() {
      const librariesSnapshot = await getDocs(collection(db, 'libraries'))
      const libraries: LibraryData[] = []
    
      for (const doc of librariesSnapshot.docs) {
        // Get collection of documents from this library
        const documentsQuery = query(
          collectionGroup(db, 'documents'),
          where('from_uid', '==', doc.id)
        )
        const documentsSnapshot = await getDocs(documentsQuery)
        const documents: DocumentData[] = []
    
        for (const document of documentsSnapshot.docs) {
          const documentData = document.data()
          const docItem: DocumentData = {
            uid: document.id,
            username: documentData.username,
            from_userid: documentData.from_userid,
            title: documentData.title,
            url: documentData.url,
            content: documentData.content,
            timestamp: documentData.timestamp.seconds
          }
          documents.push(docItem)
        }
    
        // Create library object
        const libraryData = doc.data()
        const library: LibraryData = {
          LibraryID: doc.id,
          InstitutionID: libraryData.institution_id,
          documents: documents
        }
        libraries.push(library)
      }
    
      console.log('libraryStore.fetchLibraries: ', libraries)
      this.libraries = libraries
    },
    async addDocumentToLibrary(file: File, user: UserData) {
      await withLoading.call(this, async () => {
      if (this.selectedLibrary) {
        const libraryRef = doc(db, 'libraries', this.selectedLibrary.LibraryID)
        const libraryData = await getDoc(libraryRef)
        if (libraryData.exists()) {
          // Upload file to storage and get the download URL
          const fileRef = ref(storage, `libraries/${this.selectedLibrary.LibraryID}/${file.name}`)
          await uploadBytes(fileRef, file)
          const downloadURL = await getDownloadURL(fileRef)

          // Add the document in the collection
          const currentTimestamp = Timestamp.now()
          const documentData = {
            content: null,
            username: user.username,
            title: file.name,
            from_uid: this.selectedLibrary.LibraryID,
            from_userid: user.uid,
            timestamp: currentTimestamp,
            url: downloadURL
          }
          const documentRef = await addDoc(collection(libraryRef, 'documents'), documentData)

          // Update local state
          const newDocument: DocumentData = {
            uid: documentRef.id,
            username: user.username,
            title: file.name,
            from_userid: user.uid,
            url: downloadURL,
            content: '',
            timestamp: currentTimestamp.seconds
          }
          this.selectedLibrary.documents.push(newDocument)
        }
      }},'Document added to library !')
    },
    async removeDocumentFromLibrary(document: DocumentData) {
      await withLoading.call(this, async () => {
      if (this.selectedLibrary) {
        const libraryRef = doc(db, 'libraries', this.selectedLibrary.LibraryID)
        const documentRef = doc(libraryRef, 'documents', document.uid)

        // Delete the document from Storage
        const fileRef = ref(
          storage,
          `libraries/${this.selectedLibrary.LibraryID}/${document.title}`
        )
        await deleteObject(fileRef)

        // Delete the document from Firestore
        await deleteDoc(documentRef)

        // Update local state
        this.selectedLibrary.documents = this.selectedLibrary.documents.filter(
          (doc) => doc.uid !== document.uid
        )
      }
    },'Document removed from library !')
    },
  }
})
