import { defineStore } from 'pinia'
import {
  createUserWithEmailAndPassword, getAuth,
  signInWithEmailAndPassword, signInAnonymously,
  type User,
} from 'firebase/auth'
import { doc, setDoc, getDoc, updateDoc, type DocumentSnapshot } from 'firebase/firestore'
import { generateUsername, withLoading } from '@/stores/utils'
import router from '@/router'
import { auth, db } from '@/stores/utils/firebase'
import { io } from 'socket.io-client'
import { type UserData } from '@/model/userData'



const socket = io("https://game-service-993049008220.europe-west4.run.app")
/**
 * Defines a store named 'userStore' using the 'defineStore' function from 'pinia'.
 * This store is used to manage the state and actions related to the user.
 */
export const useUserStore = defineStore('userStore', {
  state: () => {
    /**
     * Defines the initial state of the store.
     */
    return {
      user: auth.currentUser as User | null, // The current user.
      userData: null as UserData | null, // The data of the current user.
      username: '' as string, // The username of the current user.
      uid: '' as string, // The unique identifier of the current user.
      isLoggedIn: false, // A flag indicating whether the user is logged in.
      loading: false as boolean, // A flag indicating whether a loading operation is in progress.
      isAnonymous: true as boolean, // A flag indicating whether the user is anonymous.
      selectedUserData: null as UserData | null // The data of the selected user.
    }
  },
  actions: {
    /**
     * Registers a new user with the given email and password.
     * @param {string} email - The email of the user.
     * @param {string} password - The password of the user.
     */
    async registerUserWithMailAndPassword(email: string, password: string) {
      console.log(email, password)
      await withLoading.call(this, async () => {
        this.user = (await createUserWithEmailAndPassword(auth, email, password)).user
        this.username = generateUsername()
        await setDoc(doc(db, 'users', this.user.uid), {
          uid: this.user.uid,
          username: this.username,
        })
        this.isLoggedIn = true
        this.isAnonymous = false
        await router.push({ name: 'Home' })
      }, 'You have been successfully registered !')
    },
    /**
     * Logs in a user with the given email and password.
     * @param {string} email - The email of the user.
     * @param {string} password - The password of the user.
     */
    async loginUserWithMailAndPassword(email: string, password: string) {
      await withLoading.call(this, async () => {
        this.user = (await signInWithEmailAndPassword(auth, email, password)).user
        this.isLoggedIn = true
        await router.push({ name: 'Home' })
      }, 'You have been successfully signed in !')
    },
    /**
     * Loads the current user.
     */
    async loadUser() {
        console.log('Loading user...')
        this.user = getAuth().currentUser
        console.log(this.user)
        if (this.user !== null) {
          const docSnap = await getDoc(doc(db, 'users', this.user.uid)) as DocumentSnapshot<UserData>
          if (docSnap.exists()) {
            console.log('DocSnap exist !')
            this.userData = docSnap.data()
            this.username = this.userData.username
            this.isAnonymous = false
          } else {
            this.username = generateUsername()
          }
          console.log("Player Connected !")
          console.log("User id = " + this.user.uid)
          socket.emit("new player", {uid: this.user.uid, x: 512, y: 256})
          console.log("Emit new player connected !")
          console.log('loading user Data... 🏃‍♂️')
          this.isLoggedIn = true
        }
      }
    ,
    /**
     * Logs in as a guest.
     */
    async playAsGuest() {
      await withLoading.call(this, async () => {
        this.user = (await signInAnonymously(auth)).user
        this.username = generateUsername()
        await setDoc(doc(db, 'users', this.user.uid), {
          uid: this.user.uid,
          username: this.username,
          points: 0
        })
        this.isLoggedIn = true
        this.isAnonymous = false
        await router.push({ name: 'Home' })

      }, 'You have been successfully signed in !')
    },

    /**
     * Logs out the current user.
     */
    async logOutUser() {
      await withLoading.call(this, async () => {
        socket.emit("close connection", { uid: this.uid })
        await getAuth().signOut()
      }, 'You have been successfully logged out !')
    },
    /**
     * Sets the current user.
     * @param {User} user - The user to set.
     */
    setUser(user: User) {
      this.user = user
    },
    /**
     * Sets the selected user by their unique identifier.
     * @param {string} userId - The unique identifier of the user.
     */
    async setSelectedUserId(userId: string) {
      console.log('loading selected user Data... 🏃‍♂️')
      const docSnap = await getDoc(doc(db, 'users', userId)) as DocumentSnapshot<UserData>
      if (docSnap.exists()) {
        this.selectedUserData = docSnap.data()
        this.selectedUserData.uid = userId

        console.log('User data found ! ', this.selectedUserData.username)
        console.log("Selected user id ", this.selectedUserData.uid)
        console.log('loading ratings for : ', this.selectedUserData.username)
      }
    },
    /**
     * Sets the unique identifier of the current user.
     * @param {string} uid - The unique identifier to set.
     */
    setUid(uid: string) {
      this.uid = uid
    },
    /**
     * Sets whether the user is logged in.
     * @param {boolean} value - The value to set.
     */
    setIsLoggedIn(value: boolean) {
      this.isLoggedIn = value
    },
    /**
     * Sets the username of the current user.
     * @param {string} value - The username to set.
     */
    async setUsername(value: string) {
      if (this.user !== null) {
        await updateDoc(doc(db, 'users', this.user.uid), {
          username: value
        })
        this.username = value
      }

    },
    async addUserPoints(points: number) {
      if (this.user !== null) {
        await updateDoc(doc(db, 'users', this.user.uid), {
          points: this.userData!.points + points
        })
        this.userData!.points = points
      }
    },

    async addPointToUser(userId: string, points: number) {
      const docSnap = (await getDoc(doc(db, 'users', userId))).data() as UserData

      await updateDoc(doc(db, 'users', userId), {
        points: docSnap.points + points
      })
    }
  },
  getters: {}
})


